<?php

/**
 * Fired during plugin activation
 *
 * @link       https://welaunch.io/plugins
 * @since      1.0.0
 *
 * @package    WordPress_Form_Wizard
 * @subpackage WordPress_Form_Wizard/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    WordPress_Form_Wizard
 * @subpackage WordPress_Form_Wizard/includes
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Form_Wizard_Activator {

	/**
	 * Short Description. (use period)
	 *
	 * Long Description.
	 *
	 * @since    1.0.0
	 */
	public function activate() {


        if( is_multisite() ) {

            $sites = get_sites();
            if(empty($sites)) {
                return false;
            }

            foreach($sites as $site) {
                
                switch_to_blog($site->blog_id);
                
                $this->create_tables();

                restore_current_blog();
            }
            
        } else {
            $this->create_tables();    
        }

        
    }

    public function create_tables()
    {
        global $wpdb;

        $db_name = $wpdb->prefix . 'form_wizard_entries';

        $sql = "CREATE TABLE " . $db_name . " (
                `id` bigint(20) NOT NULL AUTO_INCREMENT,
                `datetime` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
                `firstname` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `lastname` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `email` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `phone` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `company` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `address_street` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `address_zip` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `address_city` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `address_country` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `message` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `privacy` varchar(10) COLLATE utf8mb4_unicode_ci NULL,
                `optional_checkbox` varchar(255) COLLATE utf8mb4_unicode_ci NULL,
                `price` varchar(50) COLLATE utf8mb4_unicode_ci NULL,
                `choices_made` text COLLATE utf8mb4_unicode_ci NULL,
                `extra_data` text COLLATE utf8mb4_unicode_ci NULL,
                `custom_data` text COLLATE utf8mb4_unicode_ci NULL,
                `post_ids` text COLLATE utf8mb4_unicode_ci NULL,
                 PRIMARY KEY (`id`))";
        
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);

        // New column extra_data
        $row = $wpdb->get_results(  "SHOW COLUMNS FROM $db_name LIKE 'extra_data'" );
        if(empty($row)){
            $wpdb->query("ALTER TABLE `" . $db_name . "` ADD `extra_data` TEXT CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci NULL DEFAULT NULL AFTER `choices_made`");
        }

        // New column custom_data
        $row = $wpdb->get_results(  "SHOW COLUMNS FROM $db_name LIKE 'custom_data'" );
        if(empty($row)){
            $wpdb->query("ALTER TABLE `" . $db_name . "` ADD `custom_data` TEXT CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci NULL DEFAULT NULL AFTER `choices_made`");
        }

        // New column post_ids
        $row = $wpdb->get_results(  "SHOW COLUMNS FROM $db_name LIKE 'post_ids'" );
        if(empty($row)){
            $wpdb->query("ALTER TABLE `" . $db_name . "` ADD `post_ids` TEXT CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci NULL DEFAULT NULL AFTER `choices_made`");
        }
    }

}
